@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2008-2017 by Andrzej Rybczak                            *
- *   electricityispower@gmail.com                                          *
+ *   Copyright (C) 2008-2021 by Andrzej Rybczak                            *
+ *   andrzej@rybczak.net                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -38,16 +38,14 @@ std::istream &operator>>(std::istream &is, LyricsFetcher_ &fetcher)
 {
 	std::string s;
 	is >> s;
-	if (s == "lyricwiki")
-		fetcher = std::make_unique<LyricwikiFetcher>();
-	else if (s == "azlyrics")
+	if (s == "azlyrics")
 		fetcher = std::make_unique<AzLyricsFetcher>();
 	else if (s == "genius")
 		fetcher = std::make_unique<GeniusFetcher>();
+	else if (s == "musixmatch")
+		fetcher = std::make_unique<MusixmatchFetcher>();
 	else if (s == "sing365")
 		fetcher = std::make_unique<Sing365Fetcher>();
-	else if (s == "lyricsmania")
-		fetcher = std::make_unique<LyricsmaniaFetcher>();
 	else if (s == "metrolyrics")
 		fetcher = std::make_unique<MetrolyricsFetcher>();
 	else if (s == "justsomelyrics")
@@ -58,6 +56,8 @@ std::istream &operator>>(std::istream &is, LyricsFetcher_ &fetcher)
 		fetcher = std::make_unique<PLyricsFetcher>();
 	else if (s == "tekstowo")
 		fetcher = std::make_unique<TekstowoFetcher>();
+	else if (s == "zeneszoveg")
+		fetcher = std::make_unique<ZeneszovegFetcher>();
 	else if (s == "internet")
 		fetcher = std::make_unique<InternetLyricsFetcher>();
 	else
@@ -85,25 +85,24 @@ LyricsFetcher::Result LyricsFetcher::fetch(const std::string &artist,
 		result.second = curl_easy_strerror(code);
 		return result;
 	}
-	
+
 	auto lyrics = getContent(regex(), data);
 
 	if (lyrics.empty() || notLyrics(data))
 	{
+		//std::cerr << "Data: " << data << "\n";
+		//std::cerr << "Empty: " << lyrics.empty() << "\n";
+		//std::cerr << "Not Lyrics: " << notLyrics(data) << "\n";
 		result.second = msgNotFound;
 		return result;
 	}
-	
+
 	data.clear();
 	for (auto it = lyrics.begin(); it != lyrics.end(); ++it)
 	{
 		postProcess(*it);
 		if (!it->empty())
-		{
 			data += *it;
-			if (it != lyrics.end()-1)
-				data += "\n\n----------\n\n";
-		}
 	}
 	
 	result.second = data;
@@ -144,66 +143,6 @@ void LyricsFetcher::postProcess(std::string &data) const
 	boost::trim(data);
 }
 
-/***********************************************************************/
-
-LyricsFetcher::Result LyricwikiFetcher::fetch(const std::string &artist,
-                                              const std::string &title)
-{
-	LyricsFetcher::Result result = LyricsFetcher::fetch(artist, title);
-	if (result.first == true)
-	{
-		result.first = false;
-		
-		std::string data;
-		CURLcode code = Curl::perform(data, result.second, "", true);
-		
-		if (code != CURLE_OK)
-		{
-			result.second = curl_easy_strerror(code);
-			return result;
-		}
-		
-		auto lyrics = getContent("<div class='lyricbox'>(.*?)</div>", data);
-		
-		if (lyrics.empty())
-		{
-			result.second = msgNotFound;
-			return result;
-		}
-		std::transform(lyrics.begin(), lyrics.end(), lyrics.begin(), unescapeHtmlUtf8);
-		bool license_restriction = std::any_of(lyrics.begin(), lyrics.end(), [](const std::string &s) {
-			return s.find("Unfortunately, we are not licensed to display the full lyrics for this song at the moment.") != std::string::npos;
-		});
-		if (license_restriction)
-		{
-			result.second = "Licence restriction";
-			return result;
-		}
-		
-		data.clear();
-		for (auto it = lyrics.begin(); it != lyrics.end(); ++it)
-		{
-			stripHtmlTags(*it);
-			boost::trim(*it);
-			if (!it->empty())
-			{
-				data += *it;
-				if (it != lyrics.end()-1)
-					data += "\n\n----------\n\n";
-			}
-		}
-		
-		result.second = data;
-		result.first = true;
-	}
-	return result;
-}
-
-bool LyricwikiFetcher::notLyrics(const std::string &data) const
-{
-	return data.find("action=edit") != std::string::npos;
-}
-
 /**********************************************************************/
 
 LyricsFetcher::Result GoogleLyricsFetcher::fetch(const std::string &artist,
@@ -238,16 +177,16 @@ LyricsFetcher::Result GoogleLyricsFetcher::fetch(const std::string &artist,
 		return result;
 	}
 
-	auto urls = getContent("<A HREF=\"(.*?)\">here</A>", data);
+	auto urls = getContent("<A HREF=\"http://www.google.com/url\\?q=(.*?)\">here</A>", data);
 
 	if (urls.empty() || !isURLOk(urls[0]))
 	{
 		result.second = msgNotFound;
 		return result;
 	}
-	
+
 	data = unescapeHtmlUtf8(urls[0]);
-	
+
 	URL = data.c_str();
 	return LyricsFetcher::fetch("", "");
 }
