@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2008-2014 by Andrzej Rybczak                            *
- *   electricityispower@gmail.com                                          *
+ *   Copyright (C) 2008-2021 by Andrzej Rybczak                            *
+ *   andrzej@rybczak.net                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -22,6 +22,7 @@
 #include <cstdlib>
 #include <algorithm>
 #include <map>
+#include <boost/regex.hpp>
 
 #include "charset.h"
 #include "mpdpp.h"
@@ -189,6 +190,11 @@ int Connection::noidle()
 	return flags;
 }
 
+void Connection::setNoidleCallback(NoidleCallback callback)
+{
+	m_noidle_callback = std::move(callback);
+}
+
 Statistics Connection::getStatistics()
 {
 	prechecks();
@@ -208,7 +214,11 @@ Status Connection::getStatus()
 void Connection::UpdateDirectory(const std::string &path)
 {
 	prechecksNoCommandsList();
-	mpd_run_update(m_connection.get(), path.c_str());
+	// Use update as mpd_run_update doesn't call mpd_response_finish if the id
+	// returned from mpd_recv_update_id is 0 which breaks mopidy.
+	mpd_send_update(m_connection.get(), path.c_str());
+	mpd_recv_update_id(m_connection.get());
+	mpd_response_finish(m_connection.get());
 	checkErrors();
 }
 
@@ -467,6 +477,14 @@ void Connection::SetVolume(unsigned vol)
 	checkErrors();
 }
 
+void Connection::ChangeVolume(int change)
+{
+	prechecksNoCommandsList();
+	mpd_run_change_volume(m_connection.get(), change);
+	checkErrors();
+}
+
+
 std::string Connection::GetReplayGainMode()
 {
 	prechecksNoCommandsList();
@@ -549,19 +567,21 @@ int Connection::AddSong(const Song &s, int pos)
 	return AddSong((!s.isFromDatabase() ? "file://" : "") + s.getURI(), pos);
 }
 
-void Connection::Add(const std::string &path)
+bool Connection::Add(const std::string &path)
 {
+	bool result;
 	prechecks();
 	if (m_command_list_active)
-		mpd_send_add(m_connection.get(), path.c_str());
+		result = mpd_send_add(m_connection.get(), path.c_str());
 	else
 	{
-		mpd_run_add(m_connection.get(), path.c_str());
+		result = mpd_run_add(m_connection.get(), path.c_str());
 		checkErrors();
 	}
+	return result;
 }
 
-bool Connection::AddRandomTag(mpd_tag_type tag, size_t number)
+bool Connection::AddRandomTag(mpd_tag_type tag, size_t number, std::mt19937 &rng)
 {
 	std::vector<std::string> tags(
 		std::make_move_iterator(GetList(tag)),
@@ -570,7 +590,7 @@ bool Connection::AddRandomTag(mpd_tag_type tag, size_t number)
 	if (number > tags.size())
 		return false;
 
-	std::random_shuffle(tags.begin(), tags.end());
+	std::shuffle(tags.begin(), tags.end(), rng);
 	auto it = tags.begin();
 	for (size_t i = 0; i < number && it != tags.end(); ++i)
 	{
@@ -588,7 +608,7 @@ bool Connection::AddRandomTag(mpd_tag_type tag, size_t number)
 	return true;
 }
 
-bool Connection::AddRandomSongs(size_t number)
+bool Connection::AddRandomSongs(size_t number, const std::string &random_exclude_pattern, std::mt19937 &rng)
 {
 	prechecksNoCommandsList();
 	std::vector<std::string> files;
@@ -609,11 +629,16 @@ bool Connection::AddRandomSongs(size_t number)
 	}
 	else
 	{
-		std::random_shuffle(files.begin(), files.end());
+		std::shuffle(files.begin(), files.end(), rng);
 		StartCommandsList();
 		auto it = files.begin();
-		for (size_t i = 0; i < number && it != files.end(); ++i, ++it)
-			AddSong(*it);
+		boost::regex re(random_exclude_pattern);
+		for (size_t i = 0; i < number && it != files.end(); ++it) {
+			if (random_exclude_pattern.empty() || !boost::regex_match((*it), re)) {
+				AddSong(*it);
+				i++;
+			}
+		}
 		CommitCommandsList();
 	}
 	return true;
@@ -630,6 +655,17 @@ void Connection::Delete(unsigned pos)
 	}
 }
 
+void Connection::DeleteRange(unsigned begin, unsigned end)
+{
+	prechecks();
+	mpd_send_delete_range(m_connection.get(), begin, end);
+	if (!m_command_list_active)
+	{
+		mpd_response_finish(m_connection.get());
+		checkErrors();
+	}
+}
+
 void Connection::PlaylistDelete(const std::string &playlist, unsigned pos)
 {
 	prechecks();
@@ -666,11 +702,12 @@ void Connection::DeletePlaylist(const std::string &name)
 	checkErrors();
 }
 
-void Connection::LoadPlaylist(const std::string &name)
+bool Connection::LoadPlaylist(const std::string &name)
 {
 	prechecksNoCommandsList();
-	mpd_run_load(m_connection.get(), name.c_str());
+	bool result = mpd_run_load(m_connection.get(), name.c_str());
 	checkErrors();
+	return result;
 }
 
 void Connection::SavePlaylist(const std::string &name)
@@ -846,7 +883,9 @@ void Connection::checkConnection() const
 void Connection::prechecks()
 {
 	checkConnection();
-	noidle();
+	int flags = noidle();
+	if (flags && m_noidle_callback)
+		m_noidle_callback(flags);
 }
 
 void Connection::prechecksNoCommandsList()
