@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2008-2017 by Andrzej Rybczak                            *
- *   electricityispower@gmail.com                                          *
+ *   Copyright (C) 2008-2021 by Andrzej Rybczak                            *
+ *   andrzej@rybczak.net                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -25,6 +25,7 @@
 #include <exception>
 #include <random>
 #include <set>
+#include <stdexcept>
 #include <vector>
 
 #include <mpd/client.h>
@@ -37,36 +38,42 @@ void checkConnectionErrors(mpd_connection *conn);
 enum PlayerState { psUnknown, psStop, psPlay, psPause };
 enum ReplayGainMode { rgmOff, rgmTrack, rgmAlbum };
 
-struct ClientError: public std::exception
+struct Error: public std::exception
 {
-	ClientError(mpd_error code_, std::string msg, bool clearable_)
-	: m_code(code_), m_msg(msg), m_clearable(clearable_) { }
-	virtual ~ClientError() noexcept { }
-	
+	Error(std::string msg, bool clearable_)
+		: m_msg(msg), m_clearable(clearable_) { }
+	virtual ~Error() noexcept { }
+
 	virtual const char *what() const noexcept { return m_msg.c_str(); }
-	mpd_error code() const { return m_code; }
 	bool clearable() const { return m_clearable; }
-	
+
 private:
-	mpd_error m_code;
 	std::string m_msg;
 	bool m_clearable;
 };
 
-struct ServerError: public std::exception
+struct ClientError: public Error
+{
+	ClientError(mpd_error code_, std::string msg, bool clearable_)
+		: Error(msg, clearable_), m_code(code_) { }
+	virtual ~ClientError() noexcept { }
+	
+	mpd_error code() const { return m_code; }
+
+private:
+	mpd_error m_code;
+};
+
+struct ServerError: public Error
 {
 	ServerError(mpd_server_error code_, std::string msg, bool clearable_)
-	: m_code(code_), m_msg(msg), m_clearable(clearable_) { }
+		: Error(msg, clearable_), m_code(code_) { }
 	virtual ~ServerError() noexcept { }
 	
-	virtual const char *what() const noexcept { return m_msg.c_str(); }
 	mpd_server_error code() const { return m_code; }
-	bool clearable() const { return m_clearable; }
-	
+
 private:
 	mpd_server_error m_code;
-	std::string m_msg;
-	bool m_clearable;
 };
 
 struct Statistics
@@ -546,15 +553,16 @@ struct Connection
 	int AddSong(const std::string &, int = -1); // returns id of added song
 	int AddSong(const Song &, int = -1); // returns id of added song
 	bool AddRandomTag(mpd_tag_type, size_t, std::mt19937 &rng);
-	bool AddRandomSongs(size_t number, std::mt19937 &rng);
-	void Add(const std::string &path);
+	bool AddRandomSongs(size_t number, const std::string &random_exclude_pattern, std::mt19937 &rng);
+	bool Add(const std::string &path);
 	void Delete(unsigned int pos);
+	void DeleteRange(unsigned begin, unsigned end);
 	void PlaylistDelete(const std::string &playlist, unsigned int pos);
 	void StartCommandsList();
 	void CommitCommandsList();
 	
 	void DeletePlaylist(const std::string &name);
-	void LoadPlaylist(const std::string &name);
+	bool LoadPlaylist(const std::string &name);
 	void SavePlaylist(const std::string &);
 	void ClearPlaylist(const std::string &playlist);
 	void AddToPlaylist(const std::string &, const Song &);
