@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2008-2014 by Andrzej Rybczak                            *
- *   electricityispower@gmail.com                                          *
+ *   Copyright (C) 2008-2021 by Andrzej Rybczak                            *
+ *   andrzej@rybczak.net                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -96,13 +96,15 @@ struct Song
 	{
 		return !(operator==(rhs));
 	}
-	
+
 	const char *c_uri() const { return m_song ? mpd_song_get_uri(m_song.get()) : ""; }
 
 	static std::string ShowTime(unsigned length);
 
 	static std::string TagsSeparator;
 
+	static bool ShowDuplicateTags;
+
 private:
 	std::shared_ptr<mpd_song> m_song;
 	size_t m_hash;
